(*
 ________                        _____________
 __  ___/______ ____   _______   ___  __/___(_)_______ ___ _____
 _____ \ _  __ `/__ | / /_  _ \  __  /   __  / __  __ `__ \_  _ \
 ____/ / / /_/ / __ |/ / /  __/  _  /    _  /  _  / / / / //  __/
 /____/  \__,_/  _____/  \___/   /_/     /_/   /_/ /_/ /_/ \___/
 ___    _________________    ________              ______
 __ |  / /__  ____/___  /    ___  __/______ ______ ___  /________
 __ | / / _  /     __  /     __  /   _  __ \_  __ \__  / __  ___/
 __ |/ /  / /___   _  /___   _  /    / /_/ // /_/ /_  /  _(__  )
 _____/   \____/   /_____/   /_/     \____/ \____/ /_/   /____/

 Delphi components with source codes (c) Jakub Dusek
 ----------------------------------------------------------------
 TSTFontsCombo & TSTFontsList 1.22
 ----------------------------------------------------------------
 Two Delphi components that displays fonts combo box or list box.
 See included help files for detail description.
 ----------------------------------------------------------------
 This source code is not public domain and can be distributed
 only with this header. Mail me some bugs or good ideas about
 new functions.
 ----------------------------------------------------------------
 http://dusek.zde.cz                            j.dusek@seznam.cz
 ________________________________________________________________

*)

unit STFonts;

{$IFNDEF VER80}
 {$IFNDEF VER90}
  {$IFNDEF VER93}
    {$IFNDEF VER100}
      {$IFNDEF VER110}
        {$DEFINE ST4}
        {$IFNDEF VER120}
           {$IFNDEF VER125}
            {$DEFINE ST5}
          {$ENDIF}
        {$ENDIF}
      {$ENDIF}
    {$ENDIF}
  {$ENDIF}
 {$ENDIF}
{$ENDIF}

interface

uses
  Windows, Messages, Controls, Classes, Graphics,
  StdCtrls, SysUtils, Printers, Forms;

const
  TestSentence      = 'The quick brown fox jumps over the lazy dog';

type
  TFontType = (TrueTypeFonts, RasterFonts, DeviceFonts, OtherFonts);

  TFontsType = set of TFontType;

  TPreviewPosition = (ppLeftTop, ppLeftBottom, ppRightTop, ppRightBottom);

  TSTHintWindow = class (THintWindow)
  protected
    procedure Paint; override;
  end;
  
  TPreview = class (TPersistent)
  private
    FColor		: TColor;
    FFontColor		: TColor;
    FFontSize		: Integer;
    FFontStyle		: TFontStyles;
    FMaxWidth		: Integer;
    FMargin		: Integer;
    FPosition		: TPreviewPosition;
    FText		: string;
    FVisible		: Boolean;
  published
    property Color	: TColor read FColor write FColor default clInfoBk;
    property FontColor	: TColor read FFontColor write FFontColor default clInfoText;
    property FontSize	: Integer read FFontSize write FFontSize default 14;
    property FontStyle	: TFontStyles read FFontStyle write FFontStyle;
    property MaxWidth	: Integer read FMaxWidth write FMaxWidth default 200;
    property Margin	: Integer read FMargin write FMargin default 2;
    property Position	: TPreviewPosition read FPosition write FPosition default ppRightTop;
    property Text	: string read FText write FText;
    property Visible	: Boolean read FVisible write FVisible default True;
  end;

  TSTFontsCombo = class (TCustomComboBox)
  private
    FAutoUpdate		: Boolean;
    FTrueTypeBmp	: TBitmap;
    FRasterBmp		: TBitmap;
    FDeviceBmp		: TBitmap;
    FFontName		: TFontName;
    FFontsType		: TFontsType;
    FNoFonts		: TStringList;
    FPreview		: TPreview;
    FHintWindow		: TSTHintWindow;
    FBitmap		: TBitmap;
    FBmpWidth		: Integer;
    procedure SetFontName (Value: TFontName);
    procedure SetFontsType (Value: TFontsType);
    procedure SetNoFonts (Value: TStringList);
    procedure UpdateHeight;
    procedure NoFontsChanged (Sender: TObject);
    procedure CMFontChanged (var Message: TMessage); message CM_FONTCHANGED;
    procedure CNCommand (var Message: TWMCommand);  message CN_COMMAND;
  protected
    procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Change; override;
  public
    constructor Create (AOwner: TComponent); override;
    procedure CreateWnd; override;
    procedure Loaded; override;
    destructor Destroy; override;
    procedure SetTrueTypeBmp (Value: TBitmap);
    procedure SetRasterBmp (Value: TBitmap);
    procedure SetDeviceBmp (Value: TBitmap);
  published
    property AutoUpdate		: Boolean read FAutoUpdate write FAutoUpdate default True;
    property FontName		: TFontName read FFontName write SetFontName;
    property FontsType		: TFontsType read FFontsType write SetFontsType default [TrueTypeFonts, RasterFonts, DeviceFonts, OtherFonts];
    property TrueTypeBmp	: TBitmap read FTrueTypeBmp write SetTrueTypeBmp;
    property RasterBmp		: TBitmap read FRasterBmp write SetRasterBmp;
    property DeviceBmp		: TBitmap read FDeviceBmp write SetDeviceBmp;
    property NoFonts		: TStringList read FNoFonts write SetNoFonts;
    property Preview		: TPreview read FPreview write FPreview;
    procedure UpdateFonts;
    {$IFDEF ST4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
    {$ENDIF}
    {$IFDEF ST5}
    property OnConTextPopup;
    {$ENDIF}
    {$IFNDEF VER90}
    property ImeMode;
    property ImeName;
    {$ENDIF}
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property DropDownCount;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

  TSTFontsList = class (TCustomListBox)
  private
    FTrueTypeBmp	: TBitmap;
    FRasterBmp		: TBitmap;
    FDeviceBmp		: TBitmap;
    FFontName		: TFontName;
    FFontsType		: TFontsType;
    FNoFonts		: TStringList;
    FPreview		: TPreview;
    FHintWindow		: TSTHintWindow;
    FBitmap		: TBitmap;
    FBmpWidth		: Integer;
    procedure SetFontName (Value: TFontName);
    procedure SetFontsType (Value: TFontsType);
    procedure SetNoFonts (Value: TStringList);
    procedure UpdateHeight;
    procedure NoFontsChanged (Sender: TObject);
    procedure CMFontChanged (var Message: TMessage); message CM_FONTCHANGED;
  protected
    procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Click; override;
  public
    constructor Create (AOwner: TComponent); override;
    procedure CreateWnd; override;
    procedure Loaded; override;
    destructor Destroy; override;
    procedure SetTrueTypeBmp (Value: TBitmap);
    procedure SetRasterBmp (Value: TBitmap);
    procedure SetDeviceBmp (Value: TBitmap);
  published
    property FontName		: TFontName read FFontName write SetFontName;
    property FontsType		: TFontsType read FFontsType write SetFontsType default [TrueTypeFonts, RasterFonts, DeviceFonts, OtherFonts];
    property TrueTypeBmp	: TBitmap read FTrueTypeBmp write SetTrueTypeBmp;
    property RasterBmp		: TBitmap read FRasterBmp write SetRasterBmp;
    property DeviceBmp		: TBitmap read FDeviceBmp write SetDeviceBmp;
    property NoFonts		: TStringList read FNoFonts write SetNoFonts;
    property Preview		: TPreview read FPreview write FPreview;
    procedure UpdateFonts;
    {$IFDEF ST4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    {$ENDIF}
    {$IFDEF ST5}
    property OnContextPopup;
    {$ENDIF}
    {$IFNDEF VER90}
    property ImeMode;
    property ImeName;
    {$ENDIF}
    property Align;
    property BorderStyle;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ExtendedSelect;
    property Font;
    property IntegralHeight;
    property MultiSelect;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

  procedure Register;
  
{$R STFonts.RES}

implementation

procedure Register;
begin
  RegisterComponents ('ST Tools', [TSTFontsCombo]);
  RegisterComponents ('ST Tools', [TSTFontsList]);
end;

function GetItemHeight (Font: TFont): Integer;
var
  DC		: HDC;
  Fnt		: HFont;
  Metrics	: TTextMetric;
begin
  DC := GetDC (0);
  try
    Fnt := SelectObject (DC, Font.Handle);
    GeTTextMetrics (DC, Metrics);
    SelectObject (DC, Fnt);
  finally
    ReleaseDC (0, DC);
  end;
  Result := Metrics.tmHeight + 1;
end;

procedure TSTHintWindow.Paint;
var
  R: TRect;
begin
  R := ClientRect;
  Inc (R.Left, 2);
  Inc (R.Top, 2);
  DrawText (Canvas.Handle, PChar(Caption), -1, R, DT_Left or DT_NOPREFIX or
            DT_WORDBREAK or DrawTextBiDiModeFlagsReadingOnly);
end;

constructor TSTFontsCombo.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  FHintWindow := TSTHintWindow.Create (Self);
  FHintWindow.Canvas.Font.Charset := DEFAULT_CHARSET;
  FTrueTypeBmp := TBitmap.Create;
  FRasterBmp := TBitmap.Create;
  FDeviceBmp := TBitmap.Create;
  FBitmap := TBitmap.Create;
  FTrueTypeBmp.Handle := LoadBitmap (HInstance, 'STTTF');
  FRasterBmp.Handle := LoadBitmap (HInstance, 'STFON');
  FDeviceBmp.Handle := LoadBitmap (HInstance, 'STPRN');
  FNoFonts := TStringList.Create;
  TStringList (FNoFonts).OnChange := NoFontsChanged;
  Style := csOwnerDrawFixed;
  Sorted := True;
  FPreview := TPreview.Create;
  FPreview.FVisible := True;
  FPreview.FColor := clInfoBk;
  FPreview.FFontColor := clInfoText;
  FPreview.FPosition := ppRightTop;
  FPreview.FText := TestSentence;
  FPreview.FFontSize := 14;
  FPreview.FMaxWidth := 200;
  FPreview.FMargin := 2;
  FFontsType := [TrueTypeFonts, RasterFonts, DeviceFonts,OtherFonts];
  FAutoUpdate := True;
  UpdateHeight;
end;

procedure TSTFontsCombo.CreateWnd;
begin
  inherited CreateWnd;
  UpdateFonts;
end;

procedure TSTFontsCombo.Loaded;
begin
  inherited Loaded;
  SetFontName (FFontName);
end;

destructor TSTFontsCombo.Destroy;
begin
  FTrueTypeBmp.Free;
  FRasterBmp.Free;
  FDeviceBmp.Free;
  FNoFonts.Free;
  FBitmap.Free;
  FHintWindow.Free;
  FPreview.Free;
  inherited Destroy;
end;

procedure TSTFontsCombo.NoFontsChanged (Sender: TObject);
begin
  UpdateFonts;
end;

procedure TSTFontsCombo.SetNoFonts (Value: TStringList);
begin
  FNoFonts.Assign (Value);
  UpdateFonts;
end;

procedure TSTFontsCombo.SetTrueTypeBmp (Value: TBitmap);
begin
  FTrueTypeBmp.Assign (Value);
  UpdateHeight;
end;

procedure TSTFontsCombo.SetRasterBmp (Value: TBitmap);
begin
  FRasterBmp.Assign (Value);
  UpdateHeight;
end;

procedure TSTFontsCombo.SetDeviceBmp (Value: TBitmap);
begin
  FDeviceBmp.Assign (Value);
  UpdateHeight;
end;

procedure TSTFontsCombo.UpdateHeight;
var i: Integer;
begin
  i := GetItemHeight (Font);
  if FTrueTypeBmp.Height > i then i := FTrueTypeBmp.Height;
  if FRasterBmp.Height > i then i := FRasterBmp.Height;
  if FDeviceBmp.Height > i then i := FDeviceBmp.Height;
  ItemHeight := i + 2;
  Height := ItemHeight;
  FBmpWidth := FTrueTypeBmp.Width;
  if FRasterBmp.Width > FBmpWidth then FBmpWidth := FRasterBmp.Width;
  if FDeviceBmp.Width > FBmpWidth then FBmpWidth := FDeviceBmp.Width;
  RecreateWnd;
end;

procedure TSTFontsCombo.CMFontChanged (var Message: TMessage);
begin
  inherited;
  UpdateHeight;
end;

procedure TSTFontsCombo.CNCommand (var Message: TWMCommand);
begin
  inherited;
  case Message.NotifyCode of
    CBN_DropDown:
    begin
      if FAutoUpdate then UpdateFonts;
    end;
  end;
end;

procedure TSTFontsCombo.Change;
begin
  FFontName := Items [ItemIndex];
  inherited Change;
end;

procedure TSTFontsCombo.SetFontName (Value: TFontName);
var Index: integer;
begin
  Index := Items.IndexOf (Value);   
  if Index <> - 1 then
  begin
    ItemIndex := Index;
    FFontName := Value;
  end;
//  if Assigned (OnChange) then OnChange (Self);
end;

procedure TSTFontsCombo.SetFontsType(Value:TFontsType);
begin
  FFontsType := Value;
  UpdateFonts;
end;

procedure TSTFontsCombo.UpdateFonts;

function IsValid (Combo: TSTFontsCombo; FontType: Integer): Boolean;
begin
  if (FontType and TrueTYPE_FONTTYPE) <> 0 then
  begin
    Result := TrueTypeFonts in Combo.FontsType;
  end
  else
    if (FontType and DEVICE_FONTTYPE) <> 0 then
    begin
      Result := DeviceFonts in Combo.FontsType;
    end
    else
      if (FontType and RASTER_FONTTYPE) <> 0 then
      begin
        Result := RasterFonts in Combo.FontsType;
      end
      else
        Result := OtherFonts in Combo.FontsType;
end;

function EnumFontsProc (var EnumLogFont: TEnumLogFont; var TextMetric: TNewTextMetric; 
                        FontType: Integer; Data: LPARAM): Integer; export; stdcall;
var
  FaceName: string;
begin
  FaceName := StrPas (EnumLogFont.elfLogFont.lfFaceName);
  with TSTFontsCombo (Data) do
    if (Items.IndexOf (FaceName) = - 1)
        and (IsValid (TSTFontsCombo (Data), FontType))
             and (NoFonts.IndexOf (FaceName) = - 1) then
    begin
      Items.AddObject (FaceName, TObject (FontType));
    end;
  Result := 1;
end;

var
  DC	: HDC;
  Old	: string;
  Index	: integer;
begin
  if not HandleAllocated then Exit;
  if Items.Count <> 0 then Old := Items [ItemIndex];
  Items.BeginUpdate;
  try
    Clear;
    DC := GetDC (0);
    try
      EnumFontFamilies (DC, nil, @EnumFontsProc, Longint (Self));
      try
        EnumFontFamilies (Printer.Handle, nil, @EnumFontsProc, Longint (Self));
      except
      end;
    finally
      ReleaseDC (0, DC);
    end;
  finally
    Items.EndUpdate;
  end;
  Index := Items.IndexOf (Old);
  if Index <> - 1 then ItemIndex := Index
  else
  begin
    if Items.Count <> 0 then
    begin
      ItemIndex := 0;
      SetFontName ( Items [ItemIndex]);
    end;
  end;
  if Items.Count <> 0 then SetFontName (Items [ItemIndex]);
end;

procedure TSTFontsCombo.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);
var FontType			: Integer;
    HintRect,ItemRect, 
    BitmapRect			: TRect;
    Pos				: TPoint;
    s				: string;
begin
  if (DroppedDown) and (FPreview.FVisible) then
  begin
    s := FPreview.FText;
    if s = '%1' then s := Items [ItemIndex];
    FHintWindow.Canvas.Font.Name := Items [ItemIndex];
    FHintWindow.Canvas.Font.Size := FPreview.FFontSize;
    FHintWindow.Canvas.Font.Color := FPreview.FFontColor;
    FHintWindow.Canvas.Font.Style := FPreview.FFontStyle;
    FHintWindow.Color := FPreview.FColor;
    HintRect := FHintWindow.CalcHintRect (FPreview.FMaxWidth, s, nil);
    case FPreview.Position of
      ppLeftTop: Pos := ClientToScreen (Point (0 - (HintRect.Right - HintRect.Left) - FPreview.FMargin, 0));
      ppLeftBottom: Pos := ClientToScreen (Point (0 - (HintRect.Right - HintRect.Left) - FPreview.FMargin,Height));
      ppRightTop: Pos := ClientToScreen (Point (Width + FPreview.FMargin, 0));
      ppRightBottom: Pos := ClientToScreen (Point (Width + FPreview. FMargin, Height));
    end;
    OffsetRect (HintRect, Pos.X, Pos.Y);
    FHintWindow.ActivateHint (HintRect, s);
  end
  else
  begin
    FHintWindow.ReleaseHandle;
  end;
  FBitmap.Assign (nil);
  Canvas.FillRect (Rect);
  FontType := Integer (Items.Objects [Index]);
  if (FontType and TrueTYPE_FONTTYPE) <> 0 then FBitmap.Assign (FTrueTypeBmp)
  else
    if (FontType and DEVICE_FONTTYPE) <> 0 then FBitmap.Assign (FDeviceBmp)
    else
      if (FontType and RASTER_FONTTYPE) <> 0 then FBitmap.Assign (FRasterBmp);
  ItemRect := Bounds (Rect.Left + 2, (Rect.Top + Rect.Bottom - FBitmap.Height) div 2, FBitmap.Width, FBitmap.Height);
  BitmapRect := Bounds (0, 0, FBitmap.Width, FBitmap.Height);
  Canvas.BrushCopy (ItemRect, FBitmap, BitmapRect, FBitmap.TransParentColor);
  Rect.Left := Rect.Left + FBmpWidth + 4;
  DrawText (Canvas.Handle, PChar (Items [Index]), Length (Items [Index]),
           Rect, DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
end;

constructor TSTFontsList.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  FHintWindow := TSTHintWindow.Create (Self);
  FHintWindow.Canvas.Font.Charset := DEFAULT_CHARSET;
  FTrueTypeBmp := TBitmap.Create;
  FRasterBmp := TBitmap.Create;
  FDeviceBmp := TBitmap.Create;
  FBitmap := TBitmap.Create;
  FTrueTypeBmp.Handle := LoadBitmap (HInstance, 'STTTF');
  FRasterBmp.Handle := LoadBitmap (HInstance, 'STFON');
  FDeviceBmp.Handle := LoadBitmap (HInstance, 'STPRN');
  FNoFonts := TStringList.Create;
  TStringList (FNoFonts).OnChange := NoFontsChanged;
  Style := lbOwnerDrawFixed;
  Sorted := True;
  FPreview := TPreview.Create;
  FPreview.FVisible := True;
  FPreview.FColor := clInfoBk;
  FPreview.FFontColor := clInfoText;
  FPreview.FPosition := ppRightTop;
  FPreview.FText := TestSentence;
  FPreview.FFontSize := 14;
  FPreview.FMaxWidth := 200;
  FPreview.FMargin := 2;
  FFontsType := [TrueTypeFonts, RasterFonts, DeviceFonts,OtherFonts];
  UpdateHeight;
end;

procedure TSTFontsList.CreateWnd;
begin
  inherited CreateWnd;
  UpdateFonts;
end;

procedure TSTFontsList.Loaded;
begin
  inherited Loaded;
  SetFontName (FFontName);
end;

destructor TSTFontsList.Destroy;
begin
  FTrueTypeBmp.Free;
  FRasterBmp.Free;
  FDeviceBmp.Free;
  FNoFonts.Free;
  FBitmap.Free;
  FHintWindow.Free;
  FPreview.Free;
  inherited Destroy;
end;

procedure TSTFontsList.NoFontsChanged (Sender: TObject);
begin
  UpdateFonts;
end;

procedure TSTFontsList.SetNoFonts (Value: TStringList);
begin
  FNoFonts.Assign (Value);
  UpdateFonts;
end;

procedure TSTFontsList.SetTrueTypeBmp (Value: TBitmap);
begin
  FTrueTypeBmp.Assign(Value);
  UpdateHeight;
end;

procedure TSTFontsList.SetRasterBmp (Value: TBitmap);
begin
  FRasterBmp.Assign (Value);
  UpdateHeight;
end;

procedure TSTFontsList.SetDeviceBmp (Value: TBitmap);
begin
  FDeviceBmp.Assign (Value);
  UpdateHeight;
end;

procedure TSTFontsList.UpdateHeight;
var i: Integer;
begin
  i := GetItemHeight (Font);
  if FTrueTypeBmp.Height > i then i := FTrueTypeBmp.Height;
  if FRasterBmp.Height > i then i := FRasterBmp.Height;
  if FDeviceBmp.Height > i then i := FDeviceBmp.Height;
  ItemHeight := i + 2;
  FBmpWidth := FTrueTypeBmp.Width;
  if FRasterBmp.Width > FBmpWidth then FBmpWidth := FRasterBmp.Width;
  if FDeviceBmp.Width > FBmpWidth then FBmpWidth := FDeviceBmp.Width;
  RecreateWnd;
end;

procedure TSTFontsList.CMFontChanged (var Message: TMessage);
begin
  inherited;
  UpdateHeight;
end;

procedure TSTFontsList.Click;
begin
  FFontName := Items [ItemIndex];
  inherited Click;
end;

procedure TSTFontsList.SetFontName (Value: TFontName);
var Index: integer;
begin
  Index := Items.IndexOf (Value);   
  if Index <> - 1 then
  begin
    ItemIndex := Index;
    FFontName := Value;
  end;
//  if Assigned (OnClick) then OnClick (Self);
end;

procedure TSTFontsList.SetFontsType (Value: TFontsType);
begin
  FFontsType := Value;
  UpdateFonts;
end;

procedure TSTFontsList.UpdateFonts;

function IsValid (Combo: TSTFontsList;FontType:Integer): Boolean;
begin
  if (FontType and TrueTYPE_FONTTYPE) <> 0 then
  begin
    Result := TrueTypeFonts in Combo.FontsType;
  end
  else
    if (FontType and DEVICE_FONTTYPE) <> 0 then
    begin
      Result := DeviceFonts in Combo.FontsType;
    end
    else
      if (FontType and RASTER_FONTTYPE) <> 0 then
      begin
        Result := RasterFonts in Combo.FontsType;
      end
      else
        Result := OtherFonts in Combo.FontsType;
end;

function EnumFontsProc (var EnumLogFont: TEnumLogFont; var TextMetric: TNewTextMetric;
                        FontType: Integer; Data: LPARAM): Integer;
  export; stdcall;
var
  FaceName  : string;
begin
  FaceName := StrPas (EnumLogFont.elfLogFont.lfFaceName);
  with TSTFontsList (Data) do
    if (Items.IndexOf (FaceName) = - 1)
        and (IsValid (TSTFontsList (Data), FontType))
             and (NoFonts.IndexOf (FaceName) = - 1) then
    begin
      Items.AddObject (FaceName, TObject (FontType));
    end;
  Result := 1;
end;

var
  DC	: HDC;
  Old	: string;
  Index	: integer;
begin
  if not HandleAllocated then Exit;
  if Items.Count <> 0 then Old := Items [ItemIndex];
  Items.BeginUpdate;
  try
    Clear;
    DC := GetDC (0);
    try
      EnumFontFamilies (DC, nil, @EnumFontsProc, Longint (Self));
      try
        EnumFontFamilies (Printer.Handle, nil, @EnumFontsProc, Longint (Self));
      except
      end;
    finally
      ReleaseDC (0, DC);
    end;
  finally
    Items.EndUpdate;
  end;
  Index := Items.IndexOf (Old);
  if Index <> - 1 then ItemIndex := Index
  else
  begin
    if Items.Count <> 0 then
    begin
      ItemIndex := 0;
      SetFontName ( Items [ItemIndex]);
    end;
  end;
  if Items.Count <> 0 then SetFontName (Items [ItemIndex]);
end;

procedure TSTFontsList.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);
var FontType, H, W	: Integer;
    HintRect, ItemRect,
    BitmapRect		: TRect;
    Pos			: TPoint;
    s			: string;
begin
  if (Focused) and (FPreview.FVisible) then
  begin
    s := FPreview.FText;
    if s = '%1' then s := Items [ItemIndex];
    FHintWindow.Canvas.Font.Name := Items [ItemIndex];
    FHintWindow.Canvas.Font.Size := FPreview.FFontSize;
    FHintWindow.Canvas.Font.Color := FPreview.FFontColor;
    FHintWindow.Canvas.Font.Style := FPreview.FFontStyle;
    FHintWindow.Color := FPreview.FColor;
    HintRect := FHintWindow.CalcHintRect (FPreview.FMaxWidth, s, nil);
    if BorderStyle = bsNone then
    begin
      W := 0;
      H := 0;
    end
    else
    begin
      W := 2;
      H := 2;
    end;
    case FPreview.Position of
      ppLeftTop: Pos := ClientToScreen (Point (0 - (HintRect.Right - HintRect.Left) - FPreview.FMargin - w,0 - h));
      ppLeftBottom: Pos := ClientToScreen (Point (0 - (HintRect.Right - HintRect.Left) - FPreview.FMargin - w,Height - (HintRect.Bottom - HintRect.Top) - H - 4));
      ppRightTop: Pos := ClientToScreen (Point (Width + FPreview.FMargin - w,0 - h));
      ppRightBottom: Pos := ClientToScreen (Point (Width + FPreview.FMargin - w, Height - (HintRect.Bottom - HintRect.Top) - H - 4));
    end;
    OffsetRect (HintRect, Pos.X, Pos.Y);
    FHintWindow.ActivateHint (HintRect, s);
  end
  else
  begin
    FHintWindow.ReleaseHandle;
  end;
  FBitmap.Assign (nil);
  Canvas.FillRect (rect);
  FontType := Integer (Items.Objects [Index]);
  if (FontType and TRUETYPE_FONTTYPE) <> 0 then FBitmap.Assign(FTrueTypeBmp)
  else
    if (FontType and DEVICE_FONTTYPE) <> 0 then FBitmap.Assign(FDeviceBmp)
    else
      if (FontType and RASTER_FONTTYPE) <> 0 then FBitmap.Assign(FRasterBmp);
  ItemRect := Bounds (Rect.Left + 2, (Rect.Top + Rect.Bottom - FBitmap.Height) div 2, FBitmap.Width, FBitmap.Height);
  BitmapRect := Bounds (0, 0, FBitmap.Width, FBitmap.Height);
  Canvas.BrushCopy (ItemRect, FBitmap, BitmapRect, FBitmap.TransParentColor);
  Rect.Left := Rect.Left + FBmpWidth + 4;
  DrawText (Canvas.Handle, PChar (Items [Index]), Length (Items [Index]),
           Rect, DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
end;

end.
